#!/bin/bash
# /opt/SPP/tools/apply_network_configurations_from_ovfTool.sh
# Verify whether the VM is deployed through OVFTool, and apply Network Configuration to the VM
# The OVFTool command needs to set ip, gateway, dns, prefix and hostname
# This script will only get executed when the property runsetup is set as true.: --extraConfig:guestinfo.runsetup=true

echo "Retrieving OVF properties"
ovf_properties=$(vmtoolsd --cmd 'info-get guestinfo.ovfEnv' 2>/dev/null)
if [ -z "$ovf_properties" ]; then
    echo "OVF Properties from guestinfo.ovfEnv is empty. Skipping IP setup."
    exit 0
fi

get_property() {
    echo "$ovf_properties" | grep -oPm1 "(?<=<Property oe:key=\"$1\" oe:value=\").*?(?=\")"
}

# Flag to control whether to run this script. When set to true, the Script will be executed.
RUNSETUP=$(vmtoolsd --cmd 'info-get guestinfo.runsetup' 2>/dev/null)
if [[ "$RUNSETUP" != "true" ]]; then
    echo "The property 'runsetup' flag is not set to 'true' Example: --extraConfig:guestinfo.runsetup=true."
    echo "Network Configuration Skipped"
    exit 0
fi

# Get properties from Guest Info
IPADDR=$(get_property ip)
GATEWAY=$(get_property gateway)
DNS=$(get_property DNS)
PREFIX=$(get_property prefix)
HOSTNAME=$(get_property hostname)

# Validate the required Properties
if [[ -z "$IPADDR" || -z "$GATEWAY" || -z "$DNS" || -z "$PREFIX" || -z "$HOSTNAME" ]]; then
    echo "Missing required OVF properties. Please provide the network properties 'ip', 'gateway', 'DNS', 'prefix' and 'hostname'."
    echo "Network Configuration Failed"
    exit 1
fi

# Wait for NetworkManager to be in Running State
for i in {1..6}; do
  nmcli networking connectivity &>/dev/null && break
  echo "Waiting for NetworkManager..."
  sleep 5
done

# Retrieve ethernet interface
NETWORK_INTERFACE=$(nmcli device status | grep 'ethernet' | awk '{print $1}' | head -n 1)
if [[ -z "$NETWORK_INTERFACE" ]]; then
    echo "No ethernet interface found."
    echo "Network Configuration Failed"
    exit 1
fi

# Remove existing connection
nmcli con delete "$NETWORK_INTERFACE" >/dev/null 2>&1

# Add new static IP config
nmcli con add type ethernet \
  ifname "$NETWORK_INTERFACE" \
  con-name static-"$NETWORK_INTERFACE" \
  ipv4.addresses "$IPADDR"/"$PREFIX" \
  ipv4.gateway "$GATEWAY" \
  ipv4.dns "$DNS" \
  ipv4.method manual \
  autoconnect yes

# Set the HostName from the property
hostnamectl set-hostname "$HOSTNAME"

# Power on the network connection
nmcli con up static-"$NETWORK_INTERFACE"

# Disable the service after first successful run
systemctl disable apply_network_configurations_from_ovfTool.service

echo "Static Network configuration applied successfully."
