import json
import subprocess
import logging
import sys

LOG_FILE = "/data/log/adminconsole/update-manager.log"
INSTALL_JSON = "/opt/SPP/update/install.json"

class LessThenFilter(logging.Filter):
    def __init__(self, level):
        self._level = level
        logging.Filter.__init__(self)

    def filter(self, rec):
        return rec.levelno < self._level

def setup():

    global logger

    logger = logging.getLogger('update')
    logger.setLevel(logging.DEBUG)

    # create a console handler
    cHandler = logging.StreamHandler(sys.stdout)
    cHandler.setLevel(logging.INFO)
    cHandler.setFormatter(logging.Formatter('%(message)s'))
    cHandler.addFilter(LessThenFilter(logging.WARNING))
    logger.addHandler(cHandler)

    # create a console handler for stderr
    ceHandler = logging.StreamHandler()
    ceHandler.setLevel(logging.ERROR)
    ceHandler.setFormatter(logging.Formatter('%(message)s'))
    # ceHandler.addFilter(LessThenFilter(logging.WARNING))
    logger.addHandler(ceHandler)

    # create a file handler
    fHandler = logging.FileHandler(LOG_FILE)
    fHandler.setLevel(logging.DEBUG)
    fHandler.setFormatter(
        logging.Formatter(
            '%(asctime)s - %(levelname)s - %(message)s',
            "%Y-%m-%d %H:%M:%S"))
    logger.addHandler(fHandler)

def load_json_file(file_path):
    """
    Load the JSON file from the specified path.
    """
    logger.info("Loading the JSON File - " + str(file_path))
    try:
        with open(file_path, 'r') as file:
            return json.load(file)
    except FileNotFoundError:
        logger.info("Error: The file " + file_path + " was not found.")
        exit(1)
    except json.JSONDecodeError as e:
        logger.info("Error: Failed to parse JSON file. " + str(e))
        exit(1)

def execute_task(task):
    """
    Execute a single task as described in the JSON object.
    """
    description = task.get("description", "No description provided")
    logger.info("Executing the Task: " + description)

    executable = task.get("exec", None)
    arguments = task.get("arguments", "")
    run_as_root = task.get("runAsRoot", False)
    timeout = task.get("timeout", None)
    abort_on_error = task.get("abortOnError", True)
    custom_error_message = task.get("customErrorMessage", "Task failed")
    display_progress = task.get("displayProgress", False)
    
    if executable:
        command = executable + " " + arguments

        if run_as_root:
            command = "sudo " + command

        logger.info("Executing command: " + command)
        
        try:
            # Use subprocess to execute the command with timeout if specified
            result = subprocess.run(command, shell=True, check=True, text=True, timeout=timeout, capture_output=True)
            if display_progress:
                logger.info("Output:\n" + result.stdout)
            if result.stderr:
                logger.info("Error Output:\n" + result.stderr)
        except subprocess.CalledProcessError as e:
            logger.info("Error: " + custom_error_message)
            if abort_on_error:
                exit(1)
        except subprocess.TimeoutExpired:
            logger.info("Error: Command timed out after " + str(timeout) + " seconds.")
            if abort_on_error:
                exit(1)
    else:
        logger.info("No executable provided for this task.")

def execute_tasks(json_data, mode):
    """
    Iterate through tasks in the JSON and execute each one.
    """
    tasks = json_data.get(mode, [])
    if not tasks:
        logger.info("No tasks found in the JSON file.")
        return
    
    for task in tasks:
        execute_task(task)

def main():
    """
    Main function to load the JSON file and execute tasks.
    """
    json_data = load_json_file(INSTALL_JSON)
    logger.info("Fetching and Executing the Tasks")
    execute_tasks(json_data, "postReboot")

if __name__ == "__main__":
    setup()
    logger.info("Starting the Post Reboot Execute Task")
    main()
    logger.info("Starting the Post Reboot Execute Task")
